@@ -2,32 +2,30 @@
 
 __author__ = """Knut Rand"""
 __email__ = "knutdrand@gmail.com"
-__version__ = '0.2.7'
+__version__ = '0.2.8'
 
 import npstructures as nps
 
-from .io.files import bnp_open as open
-from .io.files import count_entries
-from .io.indexed_files import open_indexed
-from .io.multiline_buffer import MultiLineFastaBuffer
-from .io.file_buffers import (TwoLineFastaBuffer, FastQBuffer)
-from .io.delimited_buffers import (BedBuffer, VCFBuffer, PhasedVCFMatrixBuffer, VCFMatrixBuffer,
-                                   GfaSequenceBuffer, get_bufferclass_for_datatype)
+from .io import (count_entries, open_indexed, MultiLineFastaBuffer, bnp_open,
+                 TwoLineFastaBuffer, FastQBuffer,
+                 BedBuffer, VCFBuffer, PhasedVCFMatrixBuffer, VCFMatrixBuffer,
+                 GfaSequenceBuffer, get_bufferclass_for_datatype)
 from .encodings.alphabet_encoding import (DNAEncoding, RNAENcoding, AminoAcidEncoding)
-from .encoded_array import EncodedArray, EncodedRaggedArray, as_encoded_array
-from .sequence.kmers import KmerEncoder
-from .sequence.minimizers import Minimizers
-from .sequence.position_weight_matrix import PositionWeightMatrix
-from .counter import count_encoded
+from .encoded_array import EncodedArray, EncodedRaggedArray, as_encoded_array, OneToOneEncoding, BaseEncoding, change_encoding
+from .sequence import (get_kmers, get_minimizers, get_motif_scores, count_encoded)
 from .streams import mean, bincount, histogram, streamable, quantile, MultiStream, groupby
 from .datatypes import SAMEntry, GFFEntry, Bed6
+from .io.strops import str_equal
+from . import simulate
+from . import arithmetics
+open = bnp_open
 
 
 SAMBuffer = get_bufferclass_for_datatype(SAMEntry)
 GFFBuffer = get_bufferclass_for_datatype(GFFEntry)
 Bed6Buffer = get_bufferclass_for_datatype(Bed6)
 
-__all__ = ["EncodedArray", "as_encoded_array", "EncodedRaggedArray",
+__all__ = ["EncodedArray", "EncodedRaggedArray",
            "KmerEncoder", "Minimizers", "PositionWeightMatrix", "mean",
            "bincount", "streamable", "histogram", "count_entries", "quantile",
            "BedBuffer", "VCFBuffer", "PhasedVCFMatrixBuffer", "VCFMatrixBuffer",
