@@ -2,11 +2,11 @@ import sys
 import numpy as np
 from io import FileIO
 from npstructures import RaggedArray, RaggedView, RaggedShape
+from .exceptions import FormatException
 from ..bnpdataclass import bnpdataclass
 from ..datatypes import SequenceEntry, SequenceEntryWithQuality
 from ..encoded_array import EncodedArray, EncodedRaggedArray
-from ..encodings import QualityEncoding
-
+from ..encodings import QualityEncoding, BaseEncoding
 
 NEWLINE = "\n"
 
@@ -35,6 +35,14 @@ class FileBuffer:
         self._is_validated = False
         self.size = self._data.size
 
+    def raise_if(condition, *args, **kwargs):
+        if condition:
+            raise FormatException(*args, **kwargs)
+
+    @property
+    def n_lines(self):
+        return len(self._new_lines)
+
     @classmethod
     def read_header(cls, file_object: FileIO):
         """Read the header data from the file
@@ -65,7 +73,6 @@ class FileBuffer:
             if line[0] != comment:
                 file_object.seek(-len(line), 1)
                 break
-        
 
     @classmethod
     def from_raw_buffer(cls, raw_buffer: np.ndarray, header_data=None) -> "FileBuffer":
@@ -87,22 +94,20 @@ class FileBuffer:
 
         Examples
         --------
-        
-
         """
 
         return NotImplemented
 
     @classmethod
     def from_data(cls, data: bnpdataclass) -> "FileBuffer":
-        """Create FileBuffer from a data set
+        """Create FileBuffer from a dataset
 
         Create a FileBuffer that can be written to file
 
         Parameters
         ----------
         data : npdataclass
-            Data set containing the data to be written
+            dataset containing the data to be written
 
         Returns
         -------
@@ -123,7 +128,7 @@ class FileBuffer:
         Returns
         -------
         npdataclass
-            Data set containing the data from the buffer
+            dataset containing the data from the buffer
         """
         return NotImplemented
 
@@ -149,7 +154,7 @@ class FileBuffer:
 
     @classmethod
     def contains_complete_entry(cls, chunks):
-        n_new_lines = sum(np.sum(EncodedArray(chunk) == NEWLINE) for chunk in chunks)
+        n_new_lines = sum(np.sum(EncodedArray(chunk, BaseEncoding) == NEWLINE) for chunk in chunks)
         return n_new_lines >= cls.n_lines_per_entry
 
 
@@ -159,7 +164,6 @@ class OneLineBuffer(FileBuffer):
     n_lines_per_entry = 2
     _buffer_divisor = 32
 
-
     @classmethod
     def from_raw_buffer(cls, chunk, header_data=None) -> "OneLineBuffer":
         """Create a buffer with full entries
@@ -182,7 +186,7 @@ class OneLineBuffer(FileBuffer):
 
         """
         assert header_data is None
-        chunk = EncodedArray(chunk)
+        chunk = EncodedArray(chunk, BaseEncoding)
         new_lines = np.flatnonzero(chunk == NEWLINE)
         n_lines = new_lines.size
         assert n_lines >= cls.n_lines_per_entry, "No complete entry in buffer. Try increasing chunk_size."
@@ -192,12 +196,11 @@ class OneLineBuffer(FileBuffer):
 
     def get_data(self) -> bnpdataclass:
         """Get and parse fields from each line"""
-
         self.validate_if_not()
         starts = np.insert(self._new_lines, 0, -1)
         lengths = np.diff(starts)
         self.lines = EncodedRaggedArray(self._data, RaggedShape(lengths))
-        sequences = self.lines[1 :: self.n_lines_per_entry, :-1]
+        sequences = self.lines[1::self.n_lines_per_entry, :-1]
         headers = self.lines[:: self.n_lines_per_entry, 1:-1]
         return SequenceEntry(headers, sequences)
 
@@ -223,16 +226,17 @@ class OneLineBuffer(FileBuffer):
             A ASCII encoded buffer
         """
 
-        name_lengths = entries.name.shape.lengths
-        sequence_lengths = entries.sequence.shape.lengths
+        name_lengths = entries.name.lengths
+        sequence_lengths = entries.sequence.lengths
         line_lengths = np.hstack(
             (name_lengths[:, None] + 2, sequence_lengths[:, None] + 1)
         ).ravel()
-        buf = EncodedArray(np.empty(line_lengths.sum(), dtype=np.uint8))
-        lines = RaggedArray(buf, line_lengths)
+        buf = EncodedArray(np.empty(line_lengths.sum(), dtype=np.uint8), BaseEncoding)
+        lines = EncodedRaggedArray(buf, line_lengths)
         step = cls.n_lines_per_entry
         lines[0::step, 1:-1] = entries.name
-        lines[1::step, :-1] = entries.sequence
+        lines[1::step, :-1] = EncodedRaggedArray(
+            EncodedArray(entries.sequence.encoding.decode(entries.sequence.ravel()),  entries.sequence.encoding),entries.sequence.shape)
 
         lines[0::step, 0] = ">"
 
@@ -246,7 +250,12 @@ class OneLineBuffer(FileBuffer):
             self._new_lines[self.n_lines_per_entry - 1 : -1 : self.n_lines_per_entry]
             + 1
         )
-        assert np.all(self._data[header_idxs] == self.HEADER)
+        if np.any(self._data[header_idxs] != self.HEADER) or self._data[0] != self.HEADER:
+            if self._data[0] != self.HEADER:
+                line_number = 0
+            else:
+                line_number = (np.flatnonzero(self._data[header_idxs] != self.HEADER)[0]+1)*self.n_lines_per_entry
+            raise FormatException(f"Expected header line to start with {self.HEADER}" % self._data, line_number=line_number)
         self._is_validated = True
 
 
@@ -265,12 +274,14 @@ class FastQBuffer(OneLineBuffer):
         seq_entry = super().get_data()
         quality = self.lines[3 :: self.n_lines_per_entry, :-1]
         return SequenceEntryWithQuality(
-            seq_entry.name, seq_entry.sequence, quality)
+            seq_entry.name, seq_entry.sequence,
+            #quality)
+            QualityEncoding.encode(quality))
 
     @classmethod
     def _get_line_lens(cls, entries):
-        name_lengths = entries.name.shape.lengths[:, None]
-        sequence_lengths = entries.sequence.shape.lengths[:, None]
+        name_lengths = entries.name.lengths[:, None]
+        sequence_lengths = entries.sequence.lengths[:, None]
         return (
             np.hstack(
                 (
@@ -283,16 +294,23 @@ class FastQBuffer(OneLineBuffer):
             + 1
         )
 
+    def _validate(self):
+        super()._validate()
+        if np.any(self._data[self._new_lines[1::self.n_lines_per_entry] + 1] != "+"):
+            entry_number = np.flatnonzero(self._data[self._new_lines[1::self.n_lines_per_entry] + 1] != "+")[0]
+            line_number = 2+entry_number*self.n_lines_per_entry
+            raise FormatException(f"Expected '+' at third line of entry in {self._data}", line_number=line_number)
+        
     @classmethod
     def from_data(cls, entries):
         line_lengths = cls._get_line_lens(entries)
-        buf = EncodedArray(np.empty(line_lengths.sum(), dtype=np.uint8))
+        buf = EncodedArray(np.empty(line_lengths.sum(), dtype=np.uint8), BaseEncoding)
         lines = EncodedRaggedArray(buf, line_lengths)
         step = cls.n_lines_per_entry
         lines[0::step, 1:-1] = entries.name
-        lines[1::step, :-1] = entries.sequence
+        lines[1::step, :-1] = EncodedRaggedArray(EncodedArray(entries.sequence.encoding.decode(entries.sequence.ravel()), BaseEncoding), entries.sequence.shape)
         lines[2::step, 0] = "+"
-        lines[3::step, :-1] = EncodedArray(QualityEncoding.decode(entries.quality.ravel()))
+        lines[3::step, :-1] = EncodedArray(QualityEncoding.decode(entries.quality.ravel()), BaseEncoding)
         lines[0::step, 0] = cls.HEADER
         lines[:, -1] = "\n"
 
