@@ -2,11 +2,11 @@ import os
 from itertools import chain
 import pytest
 import numpy as np
-from npstructures.testing import assert_npdataclass_equal 
 from bionumpy.io.file_buffers import FastQBuffer, TwoLineFastaBuffer
 from bionumpy.datatypes import Interval, SNP, SequenceEntry, VCFEntry
 from bionumpy.io.delimited_buffers import BedBuffer, VCFBuffer, GfaSequenceBuffer, get_bufferclass_for_datatype
 from bionumpy.io.files import bnp_open
+from bionumpy.util.testing import assert_bnpdataclass_equal
 from .buffers import fastq_buffer, twoline_fasta_buffer, bed_buffer, vcf_buffer, vcf_buffer2, gfa_sequence_buffer, combos, data
 from bionumpy.io.parser import chunk_lines
 from bionumpy.bnpdataclass import bnpdataclass
@@ -26,18 +26,28 @@ def test_buffer_read(buffer_name):
     buf, true_data, buf_type = combos[buffer_name]
     data = buf_type.from_raw_buffer(buf).get_data()
     for line, true_line in zip(data, true_data):
-        assert_npdataclass_equal(line, true_line)
+        assert_bnpdataclass_equal(line, true_line)
 
 
-@pytest.mark.parametrize("buffer_name", ["fasta", "fastq", "multiline_fasta"])  # "bed", "vcf2", "vcf", "fastq", "fasta"])
+#@pytest.mark.parametrize("buffer_name", ["fastq", "fasta", "multiline_fasta"])  # "bed", "vcf2", "vcf", "fastq", "fasta"])
+@pytest.mark.parametrize("buffer_name", ["fastq"])  # "bed", "vcf2", "vcf", "fastq", "fasta"])
 def test_buffer_write(buffer_name):
     true_buf, data, buf_type = combos[buffer_name]
     if buffer_name == "multiline_fasta":
         buf_type.n_characters_per_line = 6
     data = buf_type.dataclass.stack_with_ragged(data)
+    print("DATA")
+    print(repr(data))
     buf = buf_type.from_data(data)
+    print("Buffer")
+    print(repr(buf))
     print(buf.to_string())
     print(true_buf.to_string())
+    print("BUF")
+    print(repr(buf))
+    print("TRUE BUF")
+    print(repr(true_buf))
+    print("BUF qual")
     assert np.all(true_buf == buf)
 
 
@@ -54,7 +64,7 @@ def test_buffered_writer_ctx_manager(file, chunk_size):
     true_stream = bnp_open('example_data/reads.fq').read_chunks()
     fq_stream = bnp_open(file_path)
     for fq_item, true_item in zip(fq_stream, true_stream):
-        assert fq_item == true_item
+        assert_bnpdataclass_equal(fq_item, true_item)
 
     os.remove(file_path)
 
@@ -100,13 +110,13 @@ def test_fastq_buffer(fastq_buffer):
 
 def test_gfa_sequence_buffer(gfa_sequence_buffer):
     buf = GfaSequenceBuffer.from_raw_buffer(gfa_sequence_buffer)
-    entries = list(buf.get_sequences())
+    entries = list(buf.get_data())
     true = [
         SequenceEntry.single_entry("id1", "AACCTTGG"),
         SequenceEntry.single_entry("id4", "ACTG")
     ]
     for entry, t in zip(entries, true):
-        assert_npdataclass_equal(entry, t)
+        assert_bnpdataclass_equal(entry, t)
 
 @pytest.mark.skip("Replaced")
 def test_vcf_buffer(vcf_buffer):
@@ -134,7 +144,7 @@ def test_line_chunker(vcf_buffer2):
     lines = list(chain.from_iterable(chunk_lines([VCFBuffer.from_raw_buffer(vcf_buffer2).get_data()], n_lines=1)))
     true = data["vcf2"]
     for line, t in zip(lines, true):
-        assert_npdataclass_equal(line, t)
+        assert_bnpdataclass_equal(line, t)
 
 
 def test_read_chunk_after_read_chunks_returns_empty_dataclass():
