@@ -3,7 +3,7 @@ from npstructures.testing import assert_raggedarray_equal
 import numpy as np
 
 from bionumpy import as_encoded_array
-from bionumpy.testing import assert_encoded_raggedarray_equal
+from bionumpy.util.testing import assert_encoded_raggedarray_equal
 from bionumpy.io.strops import (int_to_str, ints_to_strings, join, split, str_to_int, str_equal, str_to_float, float_to_strings)
 
 
@@ -72,7 +72,7 @@ def test_join(strings):
     seqs = as_encoded_array(strings)
     joined = join(seqs)
     true = as_encoded_array("\t".join(strings))
-    assert np.all(joined == as_encoded_array("\t".join(strings)))
+    assert np.all(joined == true)
 
 
 def test_split(strings):
@@ -86,6 +86,10 @@ def test_str_equal(strings):
     mask = str_equal(s, "12")
     np.testing.assert_array_equal(mask, [False, False, True, False]*2)
 
+def test_str_equal_single(strings):
+    assert str_equal(strings[2], "12")
+    assert not str_equal(strings[3], "12")
+
 
 if __name__ == "__main__":
     test_split(["1", "12", "123"])
